BEGIN;

CREATE TYPE uri_table AS (scheme text,username text,passwd text,host text,port text,path text,query text,frag text);

CREATE OR REPLACE FUNCTION uri_in   (cstring)  RETURNS uri     AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;
CREATE OR REPLACE FUNCTION uri_out  (uri)      RETURNS cstring AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;
CREATE OR REPLACE FUNCTION uri_recv (internal) RETURNS uri     AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;
CREATE OR REPLACE FUNCTION uri_send (uri)      RETURNS bytea   AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;

CREATE TYPE uri ( INPUT = uri_in, OUTPUT = uri_out, RECEIVE = uri_recv, SEND = uri_send, INTERNALLENGTH = VARIABLE, STORAGE = extended, ALIGNMENT = int4 );

-- Cast functions
CREATE OR REPLACE FUNCTION uri_text     (uri)       RETURNS text      AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;
CREATE OR REPLACE FUNCTION text_uri     (text)      RETURNS uri       AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;

CREATE OR REPLACE FUNCTION uri_uri_table(uri)       RETURNS uri_table AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;
CREATE OR REPLACE FUNCTION uri_table_uri(uri_table) RETURNS uri       AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;

-- Casts
CREATE CAST ( uri AS text )      with function uri_text     ( uri )       AS IMPLICIT;
CREATE CAST ( text AS uri )      with function text_uri     ( text)       AS IMPLICIT;

CREATE CAST ( uri AS uri_table ) with function uri_uri_table( uri )       AS IMPLICIT;
CREATE CAST ( uri_table AS uri ) with function uri_table_uri( uri_table ) AS IMPLICIT;

CREATE OR REPLACE FUNCTION uri_at_uri(uri,uri)      RETURNS uri       AS 'uri' LANGUAGE 'C' IMMUTABLE;
CREATE OR REPLACE function uri_isvalid( text )      RETURNS bool      AS 'uri' LANGUAGE 'C' IMMUTABLE STRICT;

-- Extraction functions
CREATE OR REPLACE function uri_scheme( uri )   returns text AS 'uri','get_uri_scheme'   LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_user( uri )     returns text AS 'uri','get_uri_user'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_passwd( uri )   returns text AS 'uri','get_uri_passwd'   LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_host( uri )     returns text AS 'uri','get_uri_host'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_port( uri )     returns text AS 'uri','get_uri_port'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_path( uri )     returns text AS 'uri','get_uri_path'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_query( uri )    returns text AS 'uri','get_uri_query'    LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_frag( uri )     returns text AS 'uri','get_uri_frag'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_robots( uri )   returns uri  AS 'uri','get_uri_robots'   LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_root( uri )     returns uri  AS 'uri','get_uri_root'     LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_relative( uri ) returns uri  AS 'uri','get_uri_relative' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_md5( uri )      returns uri  AS 'uri','get_uri_md5'      LANGUAGE C IMMUTABLE STRICT;

-- Ordering functions
CREATE OR REPLACE function uri_eq     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_ne     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_lt     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_le     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_gt     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function uri_ge     ( uri, uri )  returns boolean AS 'uri' LANGUAGE C IMMUTABLE STRICT;
CREATE OR REPLACE function bt_uri_cmp ( uri, uri )  returns int4    AS 'uri' LANGUAGE C IMMUTABLE STRICT;

CREATE OR REPLACE function uri_hash  ( uri )      returns int4    AS 'uri' LANGUAGE C IMMUTABLE STRICT;

-- Ordering OPERATORs
CREATE OPERATOR <   ( PROCEDURE=uri_lt,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= >,   NEGATOR= >=,  RESTRICT=scalarltsel, JOIN=scalarltjoinsel );
CREATE OPERATOR <=  ( PROCEDURE=uri_le,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= >=,  NEGATOR= >,   RESTRICT=scalarltsel, JOIN=scalarltjoinsel );
CREATE OPERATOR >   ( PROCEDURE=uri_gt,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= <,   NEGATOR= <=,  RESTRICT=scalargtsel, JOIN=scalargtjoinsel );
CREATE OPERATOR >=  ( PROCEDURE=uri_ge,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= <=,  NEGATOR= <,   RESTRICT=scalargtsel, JOIN=scalargtjoinsel );
CREATE OPERATOR <>  ( PROCEDURE=uri_ne,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= <>,  NEGATOR= =,   RESTRICT=neqsel,      JOIN=neqjoinsel );
CREATE OPERATOR =   ( PROCEDURE=uri_eq,     LEFTARG=uri, RIGHTARG=uri, COMMUTATOR= =,   NEGATOR= <>,  RESTRICT=eqsel,       JOIN=eqjoinsel, HASHES, MERGES, SORT1= < );

CREATE OPERATOR class uri_ops DEFAULT FOR TYPE uri USING btree AS
    OPERATOR 1 < ,
    OPERATOR 2 <= ,
    OPERATOR 3 = ,
    OPERATOR 4 >= ,
    OPERATOR 5 > ,
    FUNCTION 1 bt_uri_cmp( uri, uri );

CREATE OPERATOR class uri_ops DEFAULT FOR TYPE uri USING hash AS
    OPERATOR 1 = ,
    FUNCTION 1 uri_hash( uri );

CREATE OPERATOR @   ( PROCEDURE=uri_at_uri, LEFTARG=uri, RIGHTARG=uri );

COMMIT;
